"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleResponse = exports.handleError = exports.default = void 0;

var _react = _interopRequireDefault(require("react"));

var _axios = _interopRequireDefault(require("axios"));

var _q3UiForms = require("q3-ui-forms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const extractResponseMeta = payload => {
  const {
    config,
    data,
    headers,
    status
  } = 'response' in payload ? payload.response : payload;
  const output = {
    data,
    headers,
    status
  };
  if (config) Object.assign(output, {
    method: config.method,
    url: config.url
  });
  return output;
};

const handleResponse = d => {
  const {
    method,
    status,
    data
  } = extractResponseMeta(d);

  const startsWith = num => String(status).startsWith(String(num));

  return {
    notify(noti) {
      if (method !== 'get' && startsWith(2) && data.message) {
        noti.onSuccess(data.message);
      } else if (startsWith(4) || startsWith(5)) {
        noti.onFail(data.message);
      }

      return this;
    },

    set() {
      return this;
    }

  };
};

exports.handleResponse = handleResponse;

const handleError = e => {
  const {
    data,
    status
  } = extractResponseMeta(e);
  return {
    notify(noti) {
      if (status === 412) {
        noti.onFail('Race condition detected (412)');
      } else if (data && data.message && status !== 404) {
        noti.onFail(data.message);
      }

      return this;
    },

    refresh() {
      return Promise.reject(e);
    }

  };
};

exports.handleError = handleError;

var _default = () => {
  const [loading, setLoading] = _react.default.useState(false);

  const noti = (0, _q3UiForms.useNotification)();

  const onRequest = request => {
    setLoading(true);
    return request;
  };

  const onRequestError = error => {
    setLoading(false);
    return error;
  };

  const onResponse = response => {
    setLoading(false);
    handleResponse(response).notify(noti).set();
    return response;
  };

  const onResponseError = error => {
    setLoading(false);

    try {
      return handleError(error).notify(noti).refresh(error);
    } catch (e) {
      return Promise.reject(error);
    }
  };

  _react.default.useEffect(() => {
    const req = _axios.default.interceptors.request.use(onRequest, onRequestError);

    const res = _axios.default.interceptors.response.use(onResponse, onResponseError);

    return () => {
      _axios.default.interceptors.request.eject(req);

      _axios.default.interceptors.response.eject(res);
    };
  }, [loading]);

  return loading;
};

exports.default = _default;