"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _axios = _interopRequireDefault(require("axios"));

var _lodash = require("lodash");

var _fp = require("lodash/fp");

var _helpers = require("../helpers");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const hasKeys = xs => (0, _lodash.isObject)(xs) && (0, _lodash.size)(Object.keys(xs));

const useRequest = ({
  namespace,
  actions: {
    decorators = {},
    next,
    store
  },
  options: {
    headers = {},
    sendUpdateAsAcknowledgement = false,
    sendUpdateAsFullReceipt = false
  }
}) => {
  const applyOptionsToUrlString = str => {
    const char = String(str).includes('?') ? '&' : '?';

    const exec = q => [str, q].join(char);

    if (sendUpdateAsAcknowledgement) return exec('acknowledge=true');
    if (sendUpdateAsFullReceipt) return exec('fullReceipt=true');
    return str;
  };

  const makeUrlPath = (a, b) => {
    let url = Array.isArray(a) ? a.filter(_helpers.isNotForwardSlash).map(_helpers.prependForwardSlash).join('') : a;
    if (Array.isArray(b)) url += `?ids[]=${b.join('&ids[]=')}`;
    return url;
  };

  const exec = ({
    callback,
    path,
    method,
    data,
    resolver
  }) => {
    const isGet = method === 'get';

    const handleError = err => Promise.reject((0, _lodash.isFunction)(callback) ? callback(err) : (0, _lodash.get)(err, 'response.data'));

    const handleNext = xs => (0, _lodash.isFunction)(next) ? next(xs).then(() => xs) : Promise.resolve(xs);

    const handleSuccess = response => (0, _lodash.isFunction)(callback) ? callback(null, response) : response;

    const invokeGetDecorator = ({
      data: response
    }) => {
      (0, _lodash.invoke)(decorators, 'get', response);
      return response;
    }; // these options are specific to Q3 subdocument routes


    const interceptResponse = response => {
      if (isGet) return response;
      if (sendUpdateAsAcknowledgement) return {
        data
      };
      if (sendUpdateAsFullReceipt) return {
        data: {
          [namespace]: (0, _lodash.get)(response, `data.full.${namespace}`),
          ...response.data
        }
      };
      return response;
    }; // mutate the data through decorators before our rest call


    if (!isGet) (0, _lodash.invoke)(decorators, method, data);
    const params = [method, path];
    if (hasKeys(data) || data instanceof FormData) params.push(data);
    if (hasKeys(headers)) params.push(headers);
    return (0, _lodash.invoke)(_axios.default, ...params).then(interceptResponse).then(invokeGetDecorator).then(handleNext).then((0, _lodash.isString)(resolver) ? payload => {
      store(resolver, payload);
      return payload;
    } : resolver).then(handleSuccess).catch(handleError);
  };

  return {
    exec,
    assembleUrl: (0, _fp.compose)(applyOptionsToUrlString, makeUrlPath),
    curry: args => data => {
      if ((0, _lodash.isObject)(data)) Object.assign(args, {
        data
      });
      return exec(args);
    }
  };
};

var _default = useRequest;
exports.default = _default;