"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.decorateDispatchReducerFn = void 0;

var _react = _interopRequireDefault(require("react"));

var _lodash = require("lodash");

var _helpers = require("../helpers");

var _reducer = _interopRequireDefault(require("./reducer"));

var _constants = require("./constants");

var _useRequest = _interopRequireDefault(require("../useRequest"));

var _useRestEffect = _interopRequireDefault(require("../useRestEffect"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const decorateDispatchReducerFn = ([state, dispatch], options = {}) => ({
  state,
  call: (type, data = {}, err = {}) => dispatch({ ...options,
    data,
    err,
    type
  }),
  // mainly the same as above just curried
  passthrough: (type, values) => payload => {
    dispatch({ ...options,
      data: values || payload,
      type
    });
    return payload;
  }
});

exports.decorateDispatchReducerFn = decorateDispatchReducerFn;

const useRest = ({
  url,
  poll,
  key,
  pluralized,
  select,
  runOnInit = false,
  decorators = {},
  ...options
}) => {
  if (!url) throw new Error('Requires a valid URL');
  const {
    state,
    call,
    passthrough
  } = decorateDispatchReducerFn(_react.default.useReducer(_reducer.default, {
    fetching: runOnInit,
    progress: 0
  }), {
    key,
    pluralized
  });
  const {
    assembleUrl,
    curry,
    exec
  } = (0, _useRequest.default)({
    namespace: pluralized,
    actions: {
      decorators,
      next: poll,
      store: call
    },
    options
  });

  const handleFetchingError = (err, data) => {
    if (!err) return data;
    call(_constants.FETCHED, null, err);
    return err;
  };

  const handleGetRequest = (query = '') => exec({
    callback: handleFetchingError,
    method: 'get',
    path: (0, _helpers.formatUrlPath)(url, // to prevent malformed requests in client apps
    // sometimes the poll method from one useRest is passed directly into another
    (0, _lodash.isString)(query) ? query : '', select),
    resolver: _constants.FETCHED
  });

  const handleGetRequestWithLoading = query => {
    call(_constants.FETCHING);
    return handleGetRequest(query).catch(() => {// noop
    });
  };

  const methods = {
    get: handleGetRequestWithLoading,
    poll: handleGetRequest,
    replace: passthrough(_constants.UPDATED),
    remove: id => curry({
      method: 'delete',
      path: assembleUrl([url, id]),
      resolver: passthrough(_constants.DELETED, {
        id
      })
    }),
    removeBulk: (ids = [], done) => curry({
      method: 'delete',
      path: assembleUrl(url, ids),
      resolver: passthrough(_constants.DELETED_MANY, {
        ids
      }),
      done
    }),
    patch: id => curry({
      method: 'patch',
      path: assembleUrl([url, id]),
      resolver: _constants.UPDATED
    }),
    patchBulk: (ids, done) => curry({
      method: 'patch',
      path: assembleUrl(url, ids),
      resolver: _constants.UPDATED,
      callback: done
    }),
    put: id => curry({
      method: 'put',
      path: assembleUrl([url, id]),
      resolver: _constants.UPDATED
    }),
    post: values => exec({
      method: 'post',
      path: assembleUrl(url),
      data: values,
      resolver: _constants.CREATED
    })
  };
  (0, _useRestEffect.default)({ ...options,
    run: handleGetRequestWithLoading,
    runOnInit,
    url
  });
  return { ...state,
    ...methods
  };
};

var _default = useRest;
exports.default = _default;